﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;

// for kinect
using Microsoft.Research.Kinect.Nui;
//using Microsoft.Speech.Recognition;
//using Microsoft.Speech.AudioFormat;
using Microsoft.Research.Kinect.Audio; 

//for osc
using Ventuz.OSC;



namespace depth
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        Runtime kinect;
    //    KinectSpeech audioKinect = null;


        public MainWindow()
        {
            InitializeComponent();

        }




        //set up writer
        private UdpWriter OSCSender = new UdpWriter("127.0.0.1", 7710);
        int player1 = 0, player2 = 0;

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // start the kinect
            kinect = Runtime.Kinects[0];


            // initalise for depth camera and skeletal tracking
            try
            {

                kinect.Initialize(RuntimeOptions.UseDepth | RuntimeOptions.UseSkeletalTracking);
            }
            catch (InvalidOperationException)
            {
                System.Windows.MessageBox.Show("Runtime initialization failed. Please make sure Kinect device is plugged in.");
                return;
            }

            // open stream
            try
            {


                kinect.DepthStream.Open(ImageStreamType.Depth, 2, ImageResolution.Resolution320x240, ImageType.Depth);
            }
            catch (InvalidOperationException)
            {
                System.Windows.MessageBox.Show("Failed to open stream. Please make sure to specify a supported image type and resolution.");
                return;
            }

            // set up the event handler
            kinect.DepthFrameReady += new EventHandler<ImageFrameReadyEventArgs>(kinect_DepthFrameReady);
            kinect.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(kinect_SkeletonFrameReady);

    //        audioKinect = new KinectSpeech();
    //        audioKinect.SaidSomething += new EventHandler<KinectSpeech.SaidSomethingArgs>(audioKinect_SaidSomething);


        }

        //void audioKinect_SaidSomething(object sender, KinectSpeech.SaidSomethingArgs e)
        //{
        //    // creat new OSC bundle    
        //    OscBundle bundle = new OscBundle();

        //    OscElement message = new OscElement("/voicecommand/" + e.word.ToString().Replace(" ", "/"));
        //    bundle.AddElement(message);
        //    OSCSender.Send(bundle);

        //}



        void kinect_SkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {
            SkeletonFrame skeletonFrame = e.SkeletonFrame;
            int iSkeleton = 0;
            foreach (SkeletonData data in skeletonFrame.Skeletons)
            {

                if (SkeletonTrackingState.Tracked == data.TrackingState)
                {
                    ++iSkeleton;
                    if (iSkeleton == 1 && player1 == 0)
                    {
                        player1 = data.TrackingID;
                        s1track.Foreground = Brushes.Green;
                        s1track.Content = "Tracking Skeleton One";

                    }
                    else if (iSkeleton == 2 && player2 == 0)
                    {
                        player2 = data.TrackingID;
                        s2track.Foreground = Brushes.Green;
                        s2track.Content = "Tracking Skeleton Two";
                    }

                    // creat new OSC bundle    
                    OscBundle bundle = new OscBundle();


                    foreach (Joint joint in data.Joints)
                    {
                        
                        // prepare the message to send
                        OscElement message = new OscElement("/joint/skeleton_" + iSkeleton.ToString() + "/"
                            + joint.ID.ToString(), joint.Position.X, joint.Position.Y, joint.Position.Z);

                        // add joint message to bundle
                        bundle.AddElement(message);
                    }

                    //send bundle
                    OSCSender.Send(bundle);

                    // update for next skeleton

                }
                if (iSkeleton == 0)
                {
                    player1 = 0;
                    s1track.Foreground = Brushes.Black;
                    s1track.Content = "Not Tracking Skeleton One";
                    player2 = 0;
                    s2track.Foreground = Brushes.Black;
                    s2track.Content = "Not Tracking Skeleton Two";
                }


            }

        }

        void kinect_DepthFrameReady(object sender, ImageFrameReadyEventArgs e)
        {
            PlanarImage depthFrame = e.ImageFrame.Image;

            // deal with the depth data 

            for (int i = 0; i < depthFrame.Bits.Length; i += 2)
            {
                int temp = (depthFrame.Bits[i + 1] << 8 | depthFrame.Bits[i]);
                temp <<= 3;
                depthFrame.Bits[i] = (byte)(temp & 0xFF);
                depthFrame.Bits[i + 1] = (byte)(temp >> 8);
            }

            depth.Source = BitmapSource.Create(depthFrame.Width, depthFrame.Height, 96, 96, PixelFormats.Gray16, null, depthFrame.Bits, depthFrame.Width * depthFrame.BytesPerPixel);
        }



        private void Window_Closed(object sender, EventArgs e)
        {
            //audioKinect.StopAudio();
            kinect.Uninitialize();
            Environment.Exit(0);
        }


    }
}
